/***************************************************************************
                          icets.cpp  -  description
                             -------------------
    begin                :    4 21:07:08 EET 2002
    copyright            : (C) 2002 by Vadim A. Khohlov
    email                : xvadim@teko.kherson.ua
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>

#include <qaccel.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmessagebox.h>

#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>

#include "icets.h"

IcetsApp::IcetsApp():
QMainWindow(),
globalDirName(qApp->argc() < 2 ? "/usr/local/lib/X11/icewm/themes" :
                                 qApp->argv()[1]),
localDirName(QDir::homeDirPath()),
bigDescription(tr("This utility allows you to switch IceWM's theme.\n\n"
                  "Using: icets [GlobalThemeDir]\n\n"
                  "If GlobalThemeDir not specified\n"
                  "\'/usr/local/lib/X11/icewm/themes\'\nwill be used."
              )),
XBA(tr("Vadim A. Khohlov")),
icetsDescription(tr("IceWM themes' swithcer " VERSION))

{
  setCaption(icetsDescription);

  QHBoxLayout *topLayout = new QHBoxLayout(this, 0, -1, "topLayout");
  CHECK_PTR(topLayout);

  lvThemesTree = new QListView(this);
  CHECK_PTR(lvThemesTree);
  lvThemesTree->setShowSortIndicator(true);
  lvThemesTree->setRootIsDecorated(true);
  topLayout->addWidget(lvThemesTree);

  QVBoxLayout *vbRight = new QVBoxLayout(topLayout, -1, "vbRight");
  CHECK_PTR(vbRight);

  previewLabel = new QLabel(bigDescription, this);
  CHECK_PTR(previewLabel);
  previewLabel->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  previewLabel->setMinimumSize(320, 200);
  previewLabel->setAlignment(AlignCenter | ExpandTabs);
  vbRight->addWidget(previewLabel);

  QHBox *hbDescription = new QHBox(this);
  CHECK_PTR(hbDescription);
  vbRight->addWidget(hbDescription);
  QLabel *lb = new QLabel(tr("Description"), hbDescription);
  CHECK_PTR(lb);
  lb->setFrameStyle(QFrame::Panel | QFrame::Raised);
  lbDescription = new QLabel(icetsDescription, hbDescription);
  CHECK_PTR(lbDescription);
  lbDescription->setFrameStyle(QFrame::Panel | QFrame::Sunken);

  QHBox *hbAuthor = new QHBox(this);
  CHECK_PTR(hbAuthor);
  vbRight->addWidget(hbAuthor);
  lb = new QLabel(tr("Author"), hbAuthor);
  CHECK_PTR(lb);
  lb->setFrameStyle(QFrame::Panel | QFrame::Raised);
  lbAuthor = new QLabel(XBA, hbAuthor);
  CHECK_PTR(lbAuthor);
  lbAuthor->setFrameStyle(QFrame::Panel | QFrame::Sunken);

  QHBox *vbButtons = new QHBox(this);
  CHECK_PTR(vbButtons);
  vbRight->addWidget(vbButtons);
  QPushButton *pbApply = new QPushButton(tr("&Apply"), vbButtons);
  CHECK_PTR(pbApply);
  QPushButton *pbOk = new QPushButton(tr("&Ok"), vbButtons);
  CHECK_PTR(pbOk);
  QPushButton *pbClose = new QPushButton(tr("&Close"), vbButtons);
  CHECK_PTR(pbClose);

  connect(pbClose, SIGNAL(clicked()), qApp, SLOT(quit()));
  connect(pbApply, SIGNAL(clicked()),       SLOT(setTheme()));
  connect(pbOk,    SIGNAL(clicked()),       SLOT(slotOk()));
  connect( lvThemesTree, SIGNAL(selectionChanged(QListViewItem*)),
	                     SLOT(slotThemeSelected( QListViewItem*)));
	
  loadThemes();

}/* IcetsApp::IcetsApp() */

IcetsApp::~IcetsApp()
{
}/*IcetsApp::~IcetsApp()*/

/** 
 * Loads themes into lvThemesTree
 */
void IcetsApp::loadThemes()
{
    QListViewItem *rootItem;
    lvThemesTree->addColumn(tr("List of Themes"));
    lvThemesTree->setFocus();

    localDirName += "/.icewm/themes";
    rootItem = new QListViewItem(lvThemesTree, tr("Global Themes"), "t");
    loadSubTreeThemes(rootItem, "g");
    rootItem = new QListViewItem(lvThemesTree, tr("Local Themes"), "t");
    loadSubTreeThemes(rootItem, "l");
    lvThemesTree->setMinimumSize(lvThemesTree->sizeHint());
}/* loadThemes() */

/** No descriptions */
void IcetsApp::loadSubTreeThemes(QListViewItem *rootItem,
                                 const QString themeType)
{
    QListViewItem *item;
	QDir dir(themeType[0] == 'g' ? globalDirName : localDirName);
    QDir subDir;

    const QFileInfoList *list = dir.entryInfoList(QDir::Dirs);
    if(!list)
	{
		QMessageBox::critical(this, "Error!", QString("Can't read ") + dir.absPath());
		return;
	} /*     if(!list) */
	QFileInfoListIterator it( *list );      // create list iterator
    QFileInfo *fi;                          // pointer for traversing
    
	/* Scaning themes*/
    while ( (fi=it.current()) )
    {
        ++it;
        if(fi->fileName()[0] == '.')
            continue;
        /* Scaning subtemes */
        subDir.setPath(fi->filePath());
        const QFileInfoList *subList = subDir.entryInfoList("*.theme",
                                                            QDir::Files);
    	if(!subList)
			continue;
		
        if(subList->count() < 1)
            continue;

        item = new QListViewItem(rootItem, fi->fileName(), themeType);

        if(subList->count() < 2)
            continue;
        QFileInfoListIterator subIt( *subList );
        QFileInfo *subFi;
        while( (subFi = subIt.current()) )
        {
            ++subIt;
            if(subFi->baseName() != "default")
                new QListViewItem(item, subFi->baseName(), "s");
        }/* while(subFi = subIt.current()) */
    }/* while ( (fi=it.current()) ) */
}/* loadSubTreeThemes(const QString dirName) */

/** No descriptions */
void IcetsApp::slotThemeSelected(QListViewItem *newItem)
{
    QString previewFile;
    QString themeFileName;
    QString previewErrMsg(tr("Can't load \n"));
    QPixmap previewPixmap;

    if(newItem->text(1)[0] == 't')
    {
        previewLabel->setText(bigDescription);
        lbAuthor->setText(XBA);
        lbDescription->setText(icetsDescription);
        return;
    }

    if(newItem->text(1)[0] != 's')  //this is not subtheme
    {
        previewFile   = getThemesDir(newItem) + "default.jpg";
        themeFileName = getThemesDir(newItem) + "default.theme";
    }/* if(newItem->text(1)[0] != 's')  //this is not subtheme */
    else    //this is subtheme
    {
        previewFile   = getThemesDir(newItem->parent()) + newItem->text(0);
        themeFileName = previewFile + ".theme";
        previewFile   += ".jpg";
    }/* else if(newItem->text(1)[0] != 's')*/

    /* Show preview image */
    QImage previewImage(previewFile);
    if(previewImage.isNull())
    {
        previewErrMsg += previewFile;
        previewLabel->setText(previewErrMsg);
    }/* if(previewImage.isNull()) */
    else
    {
        previewPixmap.convertFromImage(previewImage.smoothScale(320, 200));
        previewLabel->setPixmap(previewPixmap);
    }/* else if(previewImage.isNull()) */

    showThemeInfo(themeFileName);
}/* slotThemeSelected(QListViewItem *newItem) */

/**
 * Returns theme's directory.
 */
QString IcetsApp::getThemesDir(QListViewItem *item)
{
    QString themeDir(item->text(1)[0] == 'g' ? globalDirName :
                                               localDirName);
    themeDir += "/";
    themeDir += item->text(0);
    themeDir += "/";
    return themeDir;
}/* getThemesDir(QListViewItem *item) */

/**
 * Writes into preferences new theme and restarts IceWM fo
 * aplying changes.
 */
void IcetsApp::setTheme()
{
    QString theme("Theme = \"");
    QListViewItem *selectedTheme;
    if(!(selectedTheme = lvThemesTree->currentItem()))
        return;
    if(selectedTheme->text(1)[0] == 't')
        return;
    if(selectedTheme->text(1)[0] == 's')
    {   /* it's subtheme */
        theme += selectedTheme->parent()->text(0) + "/";
        theme += selectedTheme->text(0) + ".theme\"\n";
    }/* if(selectedTheme->text(1)[0] == 's') */
    else
        theme += selectedTheme->text(0) + "/default.theme\"\n";

    QString prefFileName(QDir::homeDirPath());
    prefFileName += "/.icewm/preferences";
    //backup copy of preferences
    QString bakPrefFileName(prefFileName);
    bakPrefFileName += ".bak";
    ::rename( QFile::encodeName(prefFileName), QFile::encodeName(bakPrefFileName));

    QFile bakFile(bakPrefFileName);
    QFile prefFile(prefFileName);
    QString s;
    QString themeStr;
    QString themeStrStart("Theme");
    QString themeStrEnd(".theme\"");

    if(bakFile.exists())
    {
        if(bakFile.open(IO_ReadOnly) && prefFile.open(IO_WriteOnly))
        {
            QTextStream bakT(&bakFile);
            QTextStream prefT(&prefFile);
            while ((s = bakT.readLine()) != QString::null)
            {
                themeStr = s.stripWhiteSpace();
                if(themeStr.find(themeStrStart) != -1 &&
                   themeStr.find(themeStrEnd) != -1) /* This is Theme string */
                    prefT << theme;
                else
                    prefT << s << '\n';
            }/* while ((s = bakT.readLine()) != QString::null) */
            bakFile.close();
        }/* if(bakFile.open(IO_ReadOnly)) */
     }/* if(bakFile.exists()) */

    //restart IceWM
    system("kill -s SIGHUP `ps -ax | awk '{print $1, $5}' | grep icewm | awk '{print $1}'`");
}/* setTheme() */

/**
 * Show theme's info into lbAuthor and lbDescription.
 */
void IcetsApp::showThemeInfo(const QString &themeFileName)
{
    QString s(tr("Can't open\n"));
    QFile themeFile(themeFileName);
    if(!themeFile.open(IO_ReadOnly))
        previewLabel->setText(s + themeFileName);
    else
    {
        QTextStream t(&themeFile);
        int f, l;
        lbAuthor->setText(tr("Unknown"));
        lbDescription->setText(tr("Unknown"));
        while ((s = t.readLine()) != QString::null)
        {
            if(s.find("ThemeAuthor") != -1)
            {
                f = s.find('\"') + 1;
                l = s.find('\"', -1);
                lbAuthor->setText(s.mid(f, l - f));
            }/* if(s.find("ThemeAuthor") != -1) */
            if(s.find("ThemeDescription") != -1)
            {
                f = s.find('\"') + 1;
                l = s.find('\"', -1);
                lbDescription->setText(s.mid(f, l - f));
            }/* if(s.find("ThemeAuthor") != -1) */
        }/* while ((s = t.readLine()) != QString::null) */
    }/* else if(!themeFile.open(IO_ReadOnly)) */
}/* showThemeInfo(const QString &themeFile) */

/** No descriptions */
void IcetsApp::slotOk()
{
    setTheme();
    qApp->quit();
}/* slotOk() */
